package auctionsim

/**
 * A Bidder represents one competitor in an auction for some resource. It is
 * not threadsafe. It is a pure value type.
 */
type Bidder struct {
    /**
     * Value stores the real value of the item being auctioned, to this bidder.
     * If the bidder won the auction at this price, they would make exactly
     * zero effective profit. It is the "rational bid" in a perfectly efficient
     * marketplace.
     *
     * A bidder that wants some nonzero profit has a lower value on the item by
     * the amount of profit they want -- that "minimum payoff" represents an
     * unspecified cost for the time and effort of doing the project at all.
     * Other costs for getting value out of the object also decrease its value.
     *
     * For example, a city to which hosting an F1 race is worth five billion
     * dollars, but would spend three and a half billion dollars setting up for
     * it, one billion dollars running it, and lose one billion dollars of tax
     * revenue due to business disruption during setup and during the event,
     * correctly values the race for negative five hundred million dollars. As
     * demonstrated by this example, a value can be negative.
     * 
     */
	Value float64

	/**
	 * Irrationality is the amount the bidder is wrong by on their belief about
	 * how much the item being auctioned is worth to them. It can be positive
	 * or negative. If it is positive, the bidder will overbid and potentially
	 * lose money by winning for more than the benefit they get from the item.
	 * If it is negative, the bidder will stop bidding too soon and potentially
	 * fail to win an auction that, had they continued bidding, would have given
	 * them the item at a profitable price.
	 */
	Irrationality float64

	/**
	 * Cash stores the bidder's maximum available cash to bid with (including
	 * any debt, liquid assets, etc. that they are willing to commit to the
	 * auction). It is a hard cap on their maximum bid.
	 *
	 * To simulate an auction without cash limits, set this to positive infinity
	 * for every bidder.
	 * 
	 * A bidder's available cash can be negative. For example, a company that
	 * cannot offer a bid to perform a job below a certain price has negative
	 * effective cash on hand repersenting the maximum price they may "pay"
	 * to win the auction -- the inverse of the minimum amount of money they
	 * are allowed to accept as compensation for the obligation being auctioned.
	 */
	Cash float64
}

/**
 * ValueBelief returns how much the bidder *thinks* the item is worth to them.
 * The actual value to them is b.Value.
func (b Bidder) ValueBelief() float64 {
	return b.Value + b.Irrationality;
}

/**
 * BidCeiling returns the highest bid this bidder would make for the item. This
 * bid might be negative. A negative winning bid is possible and ordinary; this
 * represents an auction between vendors competing to offer a service at the
 * lowest price, for example.
 */
func (b Bidder) BidCeiling() float64 {
	if b.ValueBelief() > b.Cash {
		return b.Cash
	}
	return b.ValueBelief()
}

/**
 * CashCapped returns whether this bidder *would* make a strictly higher bid
 * than their calculated ceiling if they had the money to do so, but they don't.
 */
func (b Bidder) CashCapped() bool {
	return b.ValueBelief() > b.Cash;
}

/**
 * IsZero returns whether the given Bidder is indistinguishable from one that was zero-initialized.
 */
func (b Bidder) IsZero() bool {
	return b.Value == 0 && b.Irrationality == 0 && b.Cash == 0
}
